/* ============================================================
 *
 * This file is a part of digiKam project
 * http://www.digikam.org
 *
 * Date        : 2005-24-01
 * Description : misc image filters
 *
 * Copyright (C) 2005-2010 by Gilles Caulier <caulier dot gilles at gmail dot com>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef DIMGIMAGE_FILTERS_H
#define DIMGIMAGE_FILTERS_H

// C++ includes

#include <cmath>

// Local includes

#include "digikam_export.h"
#include "globals.h"

namespace Digikam
{

class DIGIKAM_EXPORT DImgImageFilters
{
public:

    DImgImageFilters(){};
    ~DImgImageFilters(){};

private:    // Private methods used internally.

    inline int setPositionAdjusted (int Width, int Height, int X, int Y)
    {
       X = (X < 0) ? 0 : (X >= Width ) ? Width  - 1 : X;
       Y = (Y < 0) ? 0 : (Y >= Height) ? Height - 1 : Y;
       return (Y*Width*4 + 4*X);
    };

public:   // Public methods.

    void invertImage(uchar* data, int w, int h, bool sixteenBit);
    
    void gaussianBlurImage(uchar* data, int width, int height, bool sixteenBit, int radius);
    
    void sharpenImage(uchar* data, int width, int height, bool sixteenBit, int radius);
    
    void unsharpMaskImage(uchar* data, int width, int height, bool sixteenBit,
                          int radius, double amount, double threshold);
                          
    void refocusImage(uchar* data, int width, int height, bool sixteenBit, int matrixSize,
                      double radius, double gauss, double correlation, double noise);
                      
    void pixelAntiAliasing(uchar* data, int Width, int Height, double X, double Y,
                           uchar* A, uchar* R, uchar* G, uchar* B);

    void pixelAntiAliasing16(unsigned short* data, int Width, int Height, double X, double Y,
                             unsigned short* A, unsigned short* R, unsigned short* G, unsigned short* B);
};

}  // namespace Digikam

#endif /* DIMGIMAGE_FILTERS_H */
